/* CHAP06.C  (Demonstration program for Chapter 6) */
/* Copyright (c) Alan Southerton */

#include "windows.h"
#include "chap06.h"
HANDLE hTxtMem;


int PASCAL WinMain(hInstance, hPrevInstance, lpCmdLine, nCmdShow)
HANDLE hInstance;               /* current program instance */
HANDLE hPrevInstance;           /* previous program instance */
LPSTR lpCmdLine;                /* command line string */
int nCmdShow;                   /* icon or window appearance */
{
     MSG msg;
     HWND hwnd;


 /* Call window registration routine */

     if(!hPrevInstance && !InitODwindow(hInstance))
         return(FALSE);

 /* Create new instance of a window */

    if(!(hwnd = CreateODwindow(hInstance, nCmdShow)))
        return(FALSE);

 /* Message loop where Windows does it stuff */

    while (GetMessage(&msg, NULL, NULL, NULL))
    {
      TranslateMessage(&msg);
      DispatchMessage(&msg);
    }
    return(msg.wParam);
}


long FAR PASCAL WindowProcedure(hwnd, msg, wParam, lParam)
HWND hwnd;
unsigned msg;
WORD wParam;
LONG lParam;
{
    int i;
    HDC hdc;
    char buf[81];
    PAINTSTRUCT ps;
    static int Routine;

    switch(msg)
    {

       case WM_CREATE:
           InitWNDstruct(hwnd);
           break;

       case WM_PAINT:
           hdc = BeginPaint(hwnd, &ps);

           for(i=0; i < 17; i++)
               switch(Routine)
               {
                   case IDM_PRINTTXT:
                       PrintTXT(hwnd, hdc, szHomer[i]);
                       break;

                   case IDM_FORMATTXT:
                       PrintTXTfmt(hwnd, hdc, szHomer[i], DT_LEFT);
                       break;

                   case IDM_JUSTIFYTXT:
                       PrintTXTjust(hwnd, hdc, szHomer[i]);
                       break;

                   case IDM_EXTTXTOUT:
                       PrintTXText(hwnd, hdc, szHomer[i]);
                       break;

                   case IDM_TABBEDTXT:
                       strcpy(buf, "\t");
                       strcat(buf, szHomer[i]);
                       PrintTXTtab(hwnd, hdc, buf, 1);
                       break;

               }

           EndPaint(hwnd, &ps);

           if(Routine == IDM_SIMPLETXT)
           {
               xPos = yPos = 0;
               for(i=0; i < 17; i++)
                   PrintText(szHomer[i]);
           }

           SetTXTredraw(hwnd, TRUE);
           break;

        case WM_COMMAND:
            if(wParam >= IDM_CBLACK && wParam <= IDM_CBLUE)
            {
                SetTXTcolor(hwnd, wParam);
                InvalidateRect(hwnd, NULL, TRUE);
                return(NULL);
            }

            if(wParam >= IDM_BBLACK && wParam <= IDM_BBLUE)
            {
                SetTXTbkgrnd(hwnd, wParam);
                InvalidateRect(hwnd, NULL, TRUE);
                return(NULL);
            }

            if(wParam >= IDM_PRINTTXT && wParam <= IDM_TABBEDTXT)
            {
                ShowTXTcaret(hwnd, TRUE);
                Routine = wParam;
                InvalidateRect(hwnd, NULL, TRUE);
                return(NULL);
            }

            switch(wParam)
            {
                case IDM_SIMPLETXT:
                    ShowTXTcaret(hwnd, FALSE);
                    Routine = wParam;
                    InvalidateRect(hwnd, NULL, TRUE);
                    return(NULL);

                case IDM_ABOUT:
                    CallDBcode(hwnd, AboutBoxDB, "AboutBox");
                    return(NULL);

                case IDM_FORMAT:
                    CallDBcode(hwnd, FormatBoxDB, "FormatBox");
                    return(NULL);

                default:
                    return(DefWindowProc(hwnd, msg, wParam, lParam));
            }

        case WM_DESTROY:
            PostQuitMessage(0);
            break;

        default:
            return(DefWindowProc(hwnd, msg, wParam, lParam));
    }
    return(NULL);
}

/************************************************************************
 *                                                                      *
 *                         Window Intializations                        *
 *                                                                      *
 ************************************************************************/

BOOL InitODwindow(hInstance)
HANDLE hInstance;
{
    WNDCLASS  wc;

    wc.style = CS_HREDRAW | CS_VREDRAW;
    wc.lpfnWndProc = WindowProcedure;
    wc.cbClsExtra = 0;
    wc.cbWndExtra = 2 * sizeof(WORD);
    wc.hInstance = hInstance;
    wc.hIcon = LoadIcon(hInstance, "PrimerIcon");
    wc.hCursor = LoadCursor(NULL, IDC_ARROW);
    wc.hbrBackground = GetStockObject(WHITE_BRUSH);
    wc.lpszMenuName =  "Chap6menu";
    wc.lpszClassName = "SampleClass";

    return(RegisterClass(&wc));
}

HWND CreateODwindow(hInstance, nCmdShow)
HANDLE hInstance;
int nCmdShow;
{
    HWND hwnd;

    hInst = hInstance;

    hwnd = CreateWindow(
        "SampleClass",
        "Chapter 6: GDI Text",
        dwStyle,
        CW_USEDEFAULT,
        CW_USEDEFAULT,
        CW_USEDEFAULT,
        CW_USEDEFAULT,
        NULL,
        NULL,
        hInstance,
        NULL);

    if(!hwnd)
        return(FALSE);

    ShowWindow(hwnd,nCmdShow);
    UpdateWindow(hwnd);
    return(hwnd);
}

BOOL FAR PASCAL AboutBoxDB(hwndDlg, msg, wParam, lParam)
HWND hwndDlg;
unsigned msg;
WORD wParam;
LONG lParam;
{
    switch(msg)
    {
        case WM_COMMAND:
            switch(wParam)
            {
                case IDOK:
                case IDCANCEL:
                    EndDialog(hwndDlg, FALSE);
                    break;

                default:
                    return(FALSE);
            }
        default:
            return(FALSE);
    }
}

BOOL FAR PASCAL FormatBoxDB(hwndDlg, msg, wParam, lParam) /* FILENAME:
                                                           FORMTBOX.C */
HWND hwndDlg;
unsigned msg;
WORD wParam;
LONG lParam;
{
    PTXTOUT ptx;

    switch(msg)
    {
        case WM_INITDIALOG:
            ptx = TextOutput(GetParent(hwndDlg), TRUE);
            SetDlgItemInt(hwndDlg, IDD_LMARGIN, ptx->lMargin, TRUE);
            SetDlgItemInt(hwndDlg, IDD_RMARGIN, ptx->rMargin, TRUE);
            SetDlgItemInt(hwndDlg, IDD_TMARGIN, ptx->tMargin, TRUE);
            SetDlgItemInt(hwndDlg, IDD_BMARGIN, ptx->bMargin, TRUE);
            ptx = TextOutput(GetParent(hwndDlg), FALSE);
            SetFocus(GetDlgItem(hwndDlg, IDD_LMARGIN));
            return(FALSE);

        case WM_COMMAND:
            if (wParam == IDOK || wParam == IDCANCEL)
            {
                EndDialog(hwndDlg, TRUE);
                return(TRUE);
            }

            if(wParam >= IDD_LMARGIN && wParam <= IDD_BMARGIN)
                ptx = TextOutput(GetParent(hwndDlg), TRUE);

            switch(wParam)
            {
                case IDD_LMARGIN:
                    ptx->lMargin = GetDlgItemInt(hwndDlg,
                                         wParam, NULL, TRUE);
                    break;

                case IDD_RMARGIN:
                    ptx->rMargin = GetDlgItemInt(hwndDlg,
                                         wParam, NULL, TRUE);
                    break;

                case IDD_TMARGIN:
                    ptx->tMargin = GetDlgItemInt(hwndDlg,
                                         wParam, NULL, TRUE);
                    break;

                case IDD_BMARGIN:
                    ptx->bMargin = GetDlgItemInt(hwndDlg,
                                         wParam, NULL, TRUE);
                    break;

                case IDD_JUSTIFY:
                    if(HIWORD(lParam) == BN_CLICKED)
                    {
                        SendDlgItemMessage(hwndDlg, IDD_JUSTIFY,
                                                   BM_SETCHECK, 1, 0L);
                        ptx->nJustify = TRUE;
                    }
                    else
                        SendDlgItemMessage(hwndDlg, IDD_JUSTIFY,
                                                   BM_SETCHECK, 0, 0L);
                    break;

                default:
                    break;
            }

            if(wParam >= IDD_LMARGIN && wParam <= IDD_BMARGIN)
                ptx = TextOutput(GetParent(hwndDlg), FALSE);
            break;
    }
    return(FALSE);
}

/************************************************************************
 *                                                                      *
 *                        Primary Text Routines                         *
 *                                                                      *
 ************************************************************************/

void PrintText(szText)       /* FILENAME: SIMPTEXT.C */
char *szText;
{
    HWND hwnd;
    HDC hdc;
    int nCount;
    int xChar, yChar;
    PTXTOUT ptx;

 /* This routine is dependent on two global variables, namely
    the xPos and yPos variables (included specifically in the
    sample program for the routine). */

    hwnd = GetFocus();
    hdc = GetDC(hwnd);
    nCount = strlen(szText);
    GetTXTdata(hwnd, &xChar, &yChar);
    yPos += yChar;
    TextOut(hdc, xPos, yPos, szText, nCount);
    ReleaseDC(hwnd, hdc);
}

void PrintTXT(hwnd, hdc, szText)       /* FILENAME: PRINTTXT.C */
HWND hwnd;
HDC hdc;
char *szText;
{
    int nCount, nWidth;
    PTXTOUT ptx;

    ptx = TextOutput(hwnd, TRUE);
    SetTXTcolors(ptx, hdc, TRUE);
    nCount = strlen(szText);

    if(ptx->bRedraw)
    {
        ptx->xCaret = ptx->lMargin;
        ptx->yCaret = ptx->tMargin;
        ptx->bRedraw = FALSE;
    }

    TextOut(hdc, ptx->xCaret, ptx->yCaret, szText, nCount);

    ptx->xCaret = ptx->lMargin + nWidth;
    ptx->yCaret += ptx->yChar;
    ptx->xCaret = ptx->lMargin;
    SetCaretPos(ptx->xCaret, ptx->yCaret);

    TextOutput(hwnd, FALSE);
}

void PrintTXText(hwnd, hdc, szText)   /* FILENAME: PRTXTEXT.C */
HWND hwnd;
HDC hdc;
char *szText;
{
    int i, nCount, nWidth;
    RECT rc;
    PTXTOUT ptx;

 /* Adjust the right margin setting to see the effects of routine.
    Either hard code or new setting or change value in TXTOUT. */

    ptx = TextOutput(hwnd, TRUE);
    SetTXTcolors(ptx, hdc, FALSE);
    nCount = strlen(szText);

    if(ptx->bRedraw)
    {
        ptx->xCaret = ptx->lMargin;
        ptx->yCaret = ptx->tMargin;
        ptx->bRedraw = FALSE;
    }

    SetRect(&rc, ptx->xCaret, ptx->yCaret, ptx->rMargin,
                              ptx->yCaret + ptx->yChar);

    ExtTextOut(hdc, ptx->xCaret, ptx->yCaret, ETO_CLIPPED,
                    &rc, szText, nCount, NULL);

    ptx->xCaret = ptx->lMargin + nWidth;
    ptx->yCaret += ptx->yChar;
    ptx->xCaret = ptx->lMargin;
    SetCaretPos(ptx->xCaret, ptx->yCaret);

    TextOutput(hwnd, FALSE);
}

void PrintTXTfmt(hwnd, hdc, szText, wFormat)   /* FILENAME: PRTXTFMT.C */
HWND hwnd;
HDC hdc;
char *szText;
WORD wFormat;
{
    int nCount, nWidth;
    PTXTOUT ptx;

    ptx = TextOutput(hwnd, TRUE);
    SetTXTcolors(ptx, hdc, FALSE);
    nCount = strlen(szText);

    if(ptx->bRedraw)
    {
        ptx->xCaret = ptx->lMargin;
        ptx->yCaret = ptx->tMargin;
        ptx->bRedraw = FALSE;
    }
    DefineTXTrect(ptx, hdc, szText);
    DrawText(hdc, szText, nCount, &ptx->rc, wFormat);

    ptx->xCaret = ptx->lMargin + nWidth;
    ptx->yCaret += ptx->yChar;
    ptx->xCaret = ptx->lMargin;
    SetCaretPos(ptx->xCaret, ptx->yCaret);

    TextOutput(hwnd, FALSE);
}

void PrintTXTjust(hwnd, hdc, szText)    /* FILENAME: PRTXJUST.C */
HWND hwnd;
HDC hdc;
char *szText;
{
    int i, nCount, nWidth, nSpace=0;
    PTXTOUT ptx;

    ptx = TextOutput(hwnd, TRUE);
    SetTXTcolors(ptx, hdc, FALSE);

    if(ptx->bRedraw)
    {
        ptx->xCaret = ptx->lMargin;
        ptx->yCaret = ptx->tMargin;
        ptx->bRedraw = FALSE;
    }
    nCount = strlen(szText);
    for(i=0; i < nCount; i++)
        if(szText[i] == 32) nSpace++;

    nWidth=LOWORD(GetTextExtent(hdc, szText, nCount));
    SetTextJustification(hdc, ptx->rMargin-nWidth, nSpace);
    TextOut(hdc, ptx->xCaret, ptx->yCaret, szText, nCount);

    SetTextJustification(hdc, 0, nSpace);

    ptx->xCaret = ptx->lMargin + nWidth;
    ptx->yCaret += ptx->yChar;
    ptx->xCaret = ptx->lMargin;
    SetCaretPos(ptx->xCaret, ptx->yCaret);

    TextOutput(hwnd, FALSE);
}

void PrintTXTtab(hwnd, hdc, szText, Tabs)    /* FILENAME: PRTXTTAB.C */
HWND hwnd;
HDC hdc;
char *szText;
int Tabs;
{
    int i, TabStops[8];
    int nCount, nWidth;
    PTXTOUT ptx;


    ptx = TextOutput(hwnd, TRUE);
    SetTXTcolors(ptx, hdc, TRUE);
    nCount = strlen(szText);

 /* Get tab width from the TXTOUT structure */

    for(i = 0; i < 8; i++)
        TabStops[i] = ptx->TabStops;

    if(ptx->bRedraw)
    {
        ptx->xCaret = ptx->lMargin;
        ptx->yCaret = ptx->tMargin;
        ptx->bRedraw = FALSE;
    }

    TabbedTextOut(hdc, ptx->xCaret, ptx->yCaret,
                   szText, nCount, Tabs, TabStops, ptx->xCaret);

    ptx->xCaret = ptx->lMargin + nWidth;
    ptx->yCaret += ptx->yChar;
    ptx->xCaret = ptx->lMargin;
    SetCaretPos(ptx->xCaret, ptx->yCaret);

    TextOutput(hwnd, FALSE);
}

/************************************************************************
 *                                                                      *
 *                        Support Text Routines                         *
 *                                                                      *
 ************************************************************************/

void DefineTXTrect(ptx, hdc, szText)    /* FILENAME: TXTRECT.C */
PTXTOUT ptx;
HDC hdc;
char *szText;
{
     int nCount, nWidth;

     nCount = strlen(szText);
     nWidth = LOWORD(GetTextExtent(hdc, szText, nCount));

     SetRect(&ptx->rc,                           /* rect struct  */
             ptx->xCaret,                        /* left x coord */
             ptx->yCaret - ptx->yChar / 2,       /* left y coord */
             ptx->xCaret + nWidth,               /* right x coord */
             ptx->yCaret + ptx->yChar /2);       /* right y coord */
}

void GetTXTdata(hwnd, xChar, yChar)   /* FILENAME: GTTXTDAT.C */
HWND hwnd;
int *xChar;
int *yChar;
{
    HDC hdc;
    TEXTMETRIC tm;

    hdc = GetDC(hwnd);
    GetTextMetrics(hdc, &tm);
    *xChar = tm.tmAveCharWidth;
    *yChar = tm.tmHeight + tm.tmExternalLeading;
    ReleaseDC(hwnd, hdc);
}

void GetDEVinches(hwnd, xInch, yInch)  /* FILENAME: GTDVINCH.C */
HWND hwnd;
int *xInch;
int *yInch;
{
    HDC hdc;

    hdc = GetDC(hwnd);
    *xInch = GetDeviceCaps(hdc, LOGPIXELSX);
    *yInch = GetDeviceCaps(hdc, LOGPIXELSY);
    ReleaseDC(hwnd, hdc);
}

void InitWNDstruct(hwnd)   /* FILENAME: WNDSTRUCT.C */
HWND hwnd;
{
     int nSize, xChar, yChar;
     int xInch, yInch;
     PTXTOUT ptx;

     nSize = sizeof(TXTOUT);
     hTxtMem = LocalAlloc(LMEM_FIXED, nSize);
     ptx = (PTXTOUT)LocalLock(hTxtMem);
     SetWindowWord(hwnd, 0, hTxtMem);
     SetWindowWord(hwnd, 2, nSize);

     GetTXTdata(hwnd, &xChar, &yChar);
     ptx->xChar = xChar;
     ptx->yChar = yChar;
     SetTXTcaret(hwnd, xChar, yChar);

     GetDEVinches(hwnd, &xInch, &yInch);
     ptx->xInch = xInch;
     ptx->yInch = yInch;
     ptx->lMargin = xInch / 8;
     ptx->tMargin = yInch / 8;
     ptx->xCaret = ptx->lMargin;
     ptx->yCaret = ptx->tMargin;
     ptx->rMargin = 485;
     ptx->TabStops = 25;
     ptx->crColor = RGB(0,0,0);
     ptx->crBkgrnd = RGB(255,255,255);

     SetCaretPos(ptx->xCaret, ptx->yCaret);
     ptx->bRedraw = TRUE;
     LocalUnlock(hTxtMem);
}

void SetTXTbkgrnd(hwnd, color)     /* FILENAME: TXBKGRND.C */
HWND hwnd;
int color;
{

    PTXTOUT ptx;
    static COLORREF rgb[8] = { RGB(0, 0, 0), RGB(255, 255, 255),
                               RGB(255, 255, 0), RGB(255, 0, 255),
                               RGB(255, 0, 0), RGB(0, 255, 255),
                               RGB(0, 255, 0), RGB(0, 0, 255) };

    color -= BCOLORBASE;
    ptx = TextOutput(hwnd, TRUE);
    ptx->crBkgrnd = rgb[color];
    TextOutput(hwnd, FALSE);
}

void SetTXTcaret(hwnd, width, height)   /* FILENAME: SETCARET.C */
HWND hwnd;
int width;
int height;
{
    CreateCaret(hwnd, NULL, width, height);
    SetCaretPos(0, 0);
    ShowCaret(hwnd);
}

void SetTXTcolor(hwnd, color)    /* FILENAME: TXTCOLOR.C */
HWND hwnd;
int color;
{

    PTXTOUT ptx;
    static COLORREF rgb[8] = { RGB(0, 0, 0), RGB(255, 255, 255),
                               RGB(255, 255, 0), RGB(255, 0, 255),
                               RGB(255, 0, 0), RGB(0, 255, 255),
                               RGB(0, 255, 0), RGB(0, 0, 255) };

    color -= COLORBASE;
    ptx = TextOutput(hwnd, TRUE);
    ptx->crColor = rgb[color];
    TextOutput(hwnd, FALSE);
}

void SetTXTcolors(ptx, hdc, bHighlight)    /* FILENAME: SETCOLOR.C */
PTXTOUT ptx;
HDC hdc;
BOOL bHighlight;
{
    if(bHighlight)
        SetBkMode(hdc, OPAQUE);
    else
        SetBkMode(hdc, TRANSPARENT);

    SetTextColor(hdc, ptx->crColor);
    SetBkColor(hdc, ptx->crBkgrnd);
}

void SetTXTredraw(hwnd, bRedraw)   /* FILENAME: TXREDRAW.C */
HWND hwnd;
BOOL bRedraw;
{
    PTXTOUT ptx;

    hTxtMem = GetWindowWord(hwnd, 0);
    ptx = (PTXTOUT)LocalLock(hTxtMem);
    ptx->bRedraw = bRedraw;
    LocalUnlock(hTxtMem);
}

void ShowTXTcaret(hwnd, bState)     /* FILENAME: SHTXTCAR.C */
HWND hwnd;
BOOL bState;
{
    int i;
    static int nRequest;

    if(bState)
    {
        for(i=0; i<=nRequest; i++)
            ShowCaret(hwnd);
        nRequest = 0;
    }
    else
    {
        HideCaret(hwnd);
        nRequest++;
    }
}


PTXTOUT TextOutput(hwnd, bBegin)   /* FILENAME: TXOUTPUT.C */
HWND hwnd;
BOOL bBegin;
{
    PTXTOUT ptx;

    if(bBegin)
    {
        hTxtMem = GetWindowWord(hwnd, 0);
        ptx = (PTXTOUT)LocalLock(hTxtMem);
        return(ptx);
    }
    else
    {
        LocalUnlock(hTxtMem);
        hTxtMem = NULL;
        return(NULL);
    }
}

/************************************************************************
 *                                                                      *
 *                     Routines From Other Chapters                     *
 *                                                                      *
 ************************************************************************/

int CallDBcode(hwnd, lpProcAddress, szDlg)  /* CHAPTER 5: CALLDB.C */
HWND hwnd;
FARPROC lpProcAddress;
char *szDlg;
{
    int nResult;
    FARPROC lpProcBox;

    lpProcBox = MakeProcInstance(lpProcAddress, hInst);
    nResult = DialogBox(hInst, (LPSTR)szDlg, hwnd, lpProcBox);
    if(nResult == -1)
    {
        MessageBeep(0);
        MessageBeep(0);
        MessageBeep(0);
        MessageBeep(0);
        MessageBeep(0);
    }
    FreeProcInstance(lpProcBox);
    return(nResult);
}

int DisplayMBinfo(hwnd, nModal, szText)
HWND hwnd;
int nModal;
char *szText;
{
     int nOutcome;
     WORD wType = MB_OK | MB_ICONINFORMATION;

     switch(nModal)
     {
         case 1:
             wType = wType | MB_APPLMODAL;
             break;

         case 2:
             wType = wType | MB_SYSTEMMODAL;
             break;

         default:
             wType = wType | MB_TASKMODAL;
             break;
     }

     nOutcome == MessageBox(hwnd,
                            szText,
                            "Application Name",
                            wType);

     return(nOutcome);
}

BOOL MoveODcenter(hwnd, xFactor, yFactor)
HWND hwnd;
int xFactor;
int yFactor;

{
    int x, y;

    if(xFactor < 1 || yFactor < 1) return(FALSE);
    x = GetSystemMetrics(SM_CXSCREEN) / 10;
    y = GetSystemMetrics(SM_CYSCREEN) / 10;

    MoveWindow(hwnd, x * (10 - xFactor) / 2,
               y * (10 - yFactor) / 2,
               x * xFactor, y * yFactor, TRUE);

    SendMessage(hwnd, WM_SYSCOMMAND, SC_RESTORE, 0L);
    return(TRUE);
}

