/* CHAP07.C  (Chapter 7: GDI Drawing) */
/* Copyright (c) Alan Southerton */

#include "windows.h"
#include "chap07.h"


int PASCAL WinMain(hInstance, hPrevInstance, lpCmdLine, nCmdShow)
HANDLE hInstance;               /* current program instance */
HANDLE hPrevInstance;           /* previous program instance */
LPSTR lpCmdLine;                /* command line string */
int nCmdShow;                   /* icon or window appearance */
{
     MSG msg;
     HWND hwnd;


 /* Call window registration routine */

     if(!hPrevInstance && !InitODwindow(hInstance))
         return(FALSE);

 /* Create new instance of a window */

    if(!(hwnd = CreateODcenter(hInstance, nCmdShow, 500, 350 )))
        return(FALSE);

 /* Message loop where Windows does it stuff */

    while (GetMessage(&msg, NULL, NULL, NULL))
    {
      TranslateMessage(&msg);
      DispatchMessage(&msg);
    }
    return(msg.wParam);
}


long FAR PASCAL WindowProcedure(hwnd, msg, wParam, lParam)
HWND hwnd;
unsigned msg;
WORD wParam;
LONG lParam;
{
    HDC hdc;
    RECT rc;
    PAINTSTRUCT ps;
    static BOOL bMouseMove, bResize;
    static BOOL bMoveBitmap;
    static BOOL bNoCareMove;
    static POINT ptMarker;
    static POINT ptStart, ptEnd;
    static POINT ptBitmap;
    static HBITMAP hBitmap;
    static int nCounter, Routine;
    char buf[30];


    switch(msg)
    {
       case WM_CREATE:
           offRight = 40;
           offBottom = GetSystemMetrics(SM_CYMENU);
           break;

       case WM_SIZE:
           bResize = TRUE;
           break;

       case WM_LBUTTONDOWN:
           if(Routine == IDM_BMMOVE)
           {
               ptBitmap = MAKEPOINT(lParam);
               InvalidateRect(hwnd, NULL, FALSE);
               EraseWNDclient(hwnd);
               bNoCareMove = TRUE;
               break;
           }

           rgbVal = GetXYcolor(hwnd, MAKEPOINT(lParam));
           break;


       case WM_RBUTTONDOWN:
           if(Routine >= IDM_DRAWARC
                    && Routine <= IDM_DRAWTRIANGLE)
               break;

           if(GetPTrange(hwnd, MAKEPOINT(lParam)))
           {
                   SetCapture(hwnd);
                   bMouseMove = TRUE;
                   ptStart=MAKEPOINT(lParam);
                   ptEnd = ptStart;
           }
           break;

       case WM_MOUSEMOVE:
           if(!GetPTrange(hwnd, MAKEPOINT(lParam)))
               break;

           UpdateXY(hwnd, NULL, MAKEPOINT(lParam));

           if(bNoCareMove)
               break;

           switch(Routine)
           {
               case IDM_DRAWLINE:
               case IDM_DRAWLINE2:
                   if(bMouseMove)
                   {
                       ptStart = ptEnd;
                       ptEnd = MAKEPOINT(lParam);
                       InvalidateRect(hwnd, NULL, FALSE);
                   }
                   break;

               case IDM_DRAWELLIPSE:
               case IDM_DRAWCIRCLE:
               case IDM_DRAWBOX:
               case IDM_DRAWRBOX:
                   if(bMouseMove)
                   {
                       ptEnd = MAKEPOINT(lParam);
                       InvalidateRect(hwnd, NULL, FALSE);
                       UpdateXY(hwnd, NULL, MAKEPOINT(lParam));
                   }
                   break;

               default:
                   break;
           }
           break;

       case WM_RBUTTONUP:
           switch(Routine)
           {
               case IDM_DRAWARC:
               case IDM_DRAWCHORD:
               case IDM_DRAWPIE:
               case IDM_DRAWTRIANGLE:

                   if(Routine < IDM_DRAWTRIANGLE && nCounter > 3)
                   {
                       InvalidateRect(hwnd, NULL, FALSE);
                       break;
                   }

                   if(Routine == IDM_DRAWTRIANGLE && nCounter > 2)
                   {
                       InvalidateRect(hwnd, NULL, FALSE);
                       break;
                   }

                   ptMarker = MAKEPOINT(lParam);
                   DrawXYmarker(ptMarker);

                   if(Routine == IDM_DRAWARC)
                       DrawXYarc(NULL, ptMarker);
                   if(Routine == IDM_DRAWCHORD)
                       DrawXYchord(NULL, ptMarker);
                   if(Routine == IDM_DRAWPIE)
                       DrawXYpie(NULL, ptMarker);
                   if(Routine == IDM_DRAWTRIANGLE)
                       DrawXYtriangle(NULL, ptMarker);

                   nCounter++;
                   break;

               case IDM_PLINEGO:
               case IDM_PGONGO:
                   ptMarker = MAKEPOINT(lParam);
                   DrawXYmarker(ptMarker);

                   if(Routine == IDM_PLINEGO)
                       DrawXYpline(NULL, ptMarker);
                   if(Routine == IDM_PGONGO)
                       DrawXYpolygon(NULL, ptMarker);

                   break;

               default:
                   ReleaseCapture();
                   bMouseMove = FALSE;
                   ptStart.x = -1;
                   break;
           }
           break;

       case WM_PAINT:
           hdc = BeginPaint(hwnd, &ps);
           SetRect(&ps.rcPaint, ps.rcPaint.left, ps.rcPaint.top,
                                ps.rcPaint.right - offRight,
                                ps.rcPaint.bottom - offBottom);

           SetHDCpen(hdc, PS_SOLID);
           SetHDCbrush(hdc, NULL_BRUSH);

           switch(Routine)
           {
               case IDM_DRAWLINE:
                   DrawXYline(hdc, ptStart, ptEnd);
                   break;

               case IDM_DRAWLINE2:
                   DrawXYline2(hdc, ptStart, ptEnd);
                   break;

               case IDM_DRAWCIRCLE:
                   DrawXYcircle(hdc, ptStart, ptEnd);
                   break;

               case IDM_DRAWBOX:
                   DrawXYbox(hdc, ptStart, ptEnd);
                   break;

               case IDM_DRAWRBOX:
                   DrawXYrbox(hdc, ptStart, ptEnd);
                   break;

               case IDM_DRAWELLIPSE:
                   DrawXYellipse(hdc, ptStart, ptEnd);
                   break;

               case IDM_DRAWARC:
                   DrawXYarc(hdc, ptMarker);
                   break;

               case IDM_DRAWCHORD:
                   DrawXYchord(hdc, ptMarker);
                   break;

               case IDM_DRAWPIE:
                   DrawXYpie(hdc, ptMarker);
                   break;

               case IDM_PLINESTOP:
                   DrawXYpline(hdc, ptMarker);
                   break;

               case IDM_PGONSTOP:
                   DrawXYpolygon(hdc, ptMarker);
                   break;

               case IDM_DRAWTRIANGLE:
                   DrawXYtriangle(hdc, ptMarker);
                   break;
           }

           EndPaint(hwnd, &ps);
           SetHDCpen(hdc, -1);
           SetHDCbrush(hdc, -1);

           switch(Routine)
           {
                case IDM_BMDISPLAY:
                    SetRect(&rc, 10, 10, 100, 100);
                    DisplayBMload(hwnd, "bm", rc);
                    break;

                case IDM_BMMAP:
                    SetRect(&rc, 10, 10, 0, 0);
                    DisplayBMmap(hwnd, "bm", rc);
                    break;

                case IDM_BMSTRETCHMAP:
                    SetRect(&rc, 10, 10, 200, 200);
                    StretchBMmap(hwnd, "bm", rc);
                    break;

                case IDM_BMSTRETCHCLIENT:
                    ChangeCursor(hwnd, IDC_WAIT);
                    StretchBMclient(hwnd, "bm");
                    ChangeCursor(hwnd, IDC_ARROW);
                    break;

                case IDM_BMMOVE:
                   if(!bMoveBitmap)
                    {
                        ptBitmap.x = 10;
                        ptBitmap.y = 10;
                        bMoveBitmap = TRUE;
                        hBitmap = LoadBitmap(hInst, "bm");
                    }
                    DisplayBMmove(hwnd, hBitmap, ptBitmap);
                    break;

                default:
                    break;
           }


           if(bResize)
           {
               ColorRGBbar(hwnd);
               StatusBox(hwnd);
               UpdateXYwnd(hwnd, hdc);
               bResize = FALSE;
           }
           break;

        case WM_COMMAND:
            switch(wParam)
            {
                case IDM_DRAWLINE:
                case IDM_DRAWLINE2:
                case IDM_DRAWBOX:
                case IDM_DRAWRBOX:
                case IDM_DRAWELLIPSE:
                case IDM_DRAWCIRCLE:
                case IDM_DRAWARC:
                case IDM_DRAWCHORD:
                case IDM_DRAWPIE:
                case IDM_PLINEGO:
                case IDM_PGONGO:
                case IDM_DRAWTRIANGLE:
                case IDM_BMDISPLAY:
                case IDM_BMMAP:
                case IDM_BMSTRETCHMAP:
                case IDM_BMSTRETCHCLIENT:
                case IDM_BMMOVE:
                case IDM_DRAWERASE:
                    EraseWNDclient(hwnd);
                    bNoCareMove = FALSE;
                    ptStart.x = 0;
                    nCounter = 0;
                    Routine = (int)wParam;

                    if(hBitmap)
                         DeleteObject(hBitmap);

                    if(wParam == IDM_BMMOVE)
                    {
                        InvalidateRect(hwnd, NULL, FALSE);
                        return(NULL);
                    }

                    bMoveBitmap = FALSE;
                    if(wParam < IDM_BMDISPLAY)
                        return(NULL);

                    InvalidateRect(hwnd, NULL, FALSE);
                    return(NULL);


                case IDM_PLINESTOP:
                case IDM_PGONSTOP:
                    Routine = (int)wParam;
                    ptMarker.x = -1;
                    InvalidateRect(hwnd, NULL, FALSE);
                    return(NULL);

                case IDM_CENTERWND:
                    MoveODcenter(hwnd, 8, 8);
                    return(NULL);

                case IDM_ABOUT:
                    CallDBcode(hwnd, AboutBoxDB, "AboutBox");
                    return(NULL);

                default:
                    return(DefWindowProc(hwnd, msg, wParam, lParam));
            }

        case WM_DESTROY:
            DeleteObject(hBitmap);
            PostQuitMessage(0);
            break;

        default:
            return(DefWindowProc(hwnd, msg, wParam, lParam));
    }
    return(NULL);
}

/************************************************************************
 *                                                                      *
 *                         Window Intializations                        *
 *                                                                      *
 ************************************************************************/

BOOL InitODwindow(hInstance)
HANDLE hInstance;
{
    WNDCLASS  wc;

    wc.style = CS_HREDRAW | CS_VREDRAW;
    wc.lpfnWndProc = WindowProcedure;
    wc.cbClsExtra = 0;
    wc.cbWndExtra = 0;
    wc.hInstance = hInstance;
    wc.hIcon = LoadIcon(hInstance, "PrimerIcon");
    wc.hCursor = LoadCursor(NULL, IDC_ARROW);
    wc.hbrBackground = GetStockObject(WHITE_BRUSH);
    wc.lpszMenuName =  "Chap6menu";
    wc.lpszClassName = "SampleClass";

    return(RegisterClass(&wc));
}

HWND CreateODcenter(hInstance, nCmdShow, cx, cy)
HANDLE hInstance;
int nCmdShow;
int cx, cy;
{
    HWND hwnd;
    int x, y;
    int xDesktop, yDesktop;

    hInst = hInstance;

    xDesktop = GetSystemMetrics(SM_CXSCREEN);
    yDesktop = GetSystemMetrics(SM_CYSCREEN);
    x = (xDesktop - cx) / 2;
    y = (yDesktop - cy) / 2;

    hwnd = CreateWindow("SampleClass", "GDI Drawing",
                         WS_OVERLAPPEDWINDOW,
                         x, y, cx, cy, NULL,
                         NULL, hInstance, NULL);

    if(!hwnd) return(FALSE);

    ShowWindow(hwnd,nCmdShow);
    UpdateWindow(hwnd);
    return(hwnd);
}



BOOL FAR PASCAL AboutBoxDB(hwndDlg, msg, wParam, lParam)
HWND hwndDlg;
unsigned msg;
WORD wParam;
LONG lParam;
{
    switch(msg)
    {
        case WM_COMMAND:
            switch(wParam)
            {
                case IDOK:
                case IDCANCEL:
                    EndDialog(hwndDlg, FALSE);
                    break;

                default:
                    return(FALSE);
            }
        default:
            return(FALSE);
    }
}

/************************************************************************
 *                                                                      *
 *                     Routines From Other Chapters                     *
 *                                                                      *
 ************************************************************************/

int CallDBcode(hwnd, lpProcAddress, szDlg)  /* CHAPTER 5: CALLDB.C */
HWND hwnd;
FARPROC lpProcAddress;
char *szDlg;
{
    int nResult;
    FARPROC lpProcBox;

    lpProcBox = MakeProcInstance(lpProcAddress, hInst);
    nResult = DialogBox(hInst, (LPSTR)szDlg, hwnd, lpProcBox);
    FreeProcInstance(lpProcBox);
    return(nResult);
}

int DisplayMBinfo(hwnd, nModal, szText)
HWND hwnd;
int nModal;
char *szText;
{
     int nOutcome;
     WORD wType = MB_OK | MB_ICONINFORMATION;

     switch(nModal)
     {
         case 1:
             wType = wType | MB_APPLMODAL;
             break;

         case 2:
             wType = wType | MB_SYSTEMMODAL;
             break;

         default:
             wType = wType | MB_TASKMODAL;
             break;
     }

     nOutcome == MessageBox(hwnd,
                            szText,
                            "Application Name",
                            wType);

     return(nOutcome);
}

BOOL MoveODcenter(hwnd, xFactor, yFactor)
HWND hwnd;
int xFactor;
int yFactor;

{
    int x, y;

    if(xFactor < 1 || yFactor < 1) return(FALSE);
    x = GetSystemMetrics(SM_CXSCREEN) / 10;
    y = GetSystemMetrics(SM_CYSCREEN) / 10;

    MoveWindow(hwnd, x * (10 - xFactor) / 2,
               y * (10 - yFactor) / 2,
               x * xFactor, y * yFactor, TRUE);

    SendMessage(hwnd, WM_SYSCOMMAND, SC_RESTORE, 0L);
    return(TRUE);
}

/************************************************************************
 *                                                                      *
 *                        Miscellaneous Routines                        *
 *                                                                      *
 ************************************************************************/

HCURSOR ChangeCursor(hwnd, lpName)      /* FILENAME: CHNGCUR.C */
HWND hwnd;
LPSTR lpName;
{
     HCURSOR hCursor;

     hCursor = LoadCursor(NULL, lpName);
     SetCursor(hCursor);
     return(hCursor);
}

void CustomWNDrect(hwnd, lpRect)    /* FILENAME: CUSWNDREC.C */
HWND hwnd;
LPRECT lpRect;
{
    GetClientRect(hwnd, lpRect);
    lpRect->left += offLeft;
    lpRect->top = offTop;
    lpRect->right -= offRight;
    lpRect->bottom -= offBottom;
}

void EraseWNDclient(hwnd)       /* FILENAME: ERASEWND.C */
HWND hwnd;
{
    HDC hdc;
    RECT rc;

    hdc = GetDC(hwnd);
    GetClientRect(hwnd, &rc);
    rc.left += offLeft;
    rc.top += offTop;
    rc.right -= offRight;
    rc.bottom -= offBottom;
    FillRect(hdc, &rc, GetStockObject(WHITE_BRUSH));
    ReleaseDC(hwnd, hdc);
}

void SetHDCpen(hdc, nType)       /* FILENAME: SETPEN.C */
HDC hdc;
int nType;
{
     static HPEN hPen;

     if(nType != -1)
     {
         hPen = CreatePen(nType, 1, rgbVal);
         SelectObject(hdc, hPen);
     }
     else
     {
         DeleteObject(hPen);
         hPen = NULL;
     }
}

void SetHDCbrush(hdc, nType)     /* FILENAME: SETBRUSH.C */
HDC hdc;
int nType;
{
     static HBRUSH hBrush;

     if(nType != -1)
     {
         hBrush = GetStockObject(nType);
         SelectObject(hdc, hBrush);
     }
     else
     {
         DeleteObject(hBrush);
         hBrush = NULL;
     }
}

void DrawXYmarker(pt)     /* DRWMARKR.C */
POINT pt;
{
    HDC hdc;
    int nPixel;

    hdc = GetDC(GetFocus());
    nPixel = GetDeviceCaps(hdc, LOGPIXELSY) / 12;
    MoveTo(hdc, pt.x, pt.y - nPixel);
    LineTo(hdc, pt.x, pt.y + nPixel);
    nPixel = GetDeviceCaps(hdc, LOGPIXELSX) / 12;
    MoveTo(hdc, pt.x - nPixel, pt.y);
    LineTo(hdc, pt.x + nPixel, pt.y);
    ReleaseDC(GetFocus(), hdc);
}



/************************************************************************
 *                                                                      *
 *                        Primitive Routines                            *
 *                                                                      *
 ************************************************************************/


BOOL DrawXYline(hdc, ptStart, ptEnd)   /* FILENAME: DRWLINE.C */
HDC hdc;
POINT ptStart;
POINT ptEnd;
{
    int i;
    static int n;
    static POINT pt[500];

    if(ptStart.x != -1)
    {
        if(n > 500)
        {
            memset(pt, 0, sizeof(pt));
            n = 0;
            DisplayMBinfo(GetFocus(), 1,
                          "Drawing array will now start over.");
            return(FALSE);
        }

        pt[n].x = ptStart.x;
        pt[n].y = ptStart.y;
        pt[n+1].x = ptEnd.x;
        pt[n+1].y = ptEnd.y;
        MoveTo(hdc, ptStart.x, ptStart.y);
        LineTo(hdc, ptEnd.x, ptEnd.y);
        n+=2;
        return(TRUE);
    }
    else
    {
        n-=2;
        for(i=0; i < n; i+=2)
        {
            MoveTo(hdc, pt[i].x, pt[i].y);
            LineTo(hdc, pt[i+1].x, pt[i+1].y);
        }
        return(TRUE);
    }
}

BOOL DrawXYline2(hdc, ptStart, ptEnd)    /* FILENAME: DRWLINE2.C */
HDC hdc;
POINT ptStart;
POINT ptEnd;
{
    static int n;
    LIST *pt;
    static LIST *head;

    if(ptStart.x != -1)
    {
        if(!(pt = (LIST *)LocalAlloc(LMEM_FIXED
                         | LMEM_ZEROINIT, sizeof(LIST))))

            DisplayMBinfo(GetFocus(), 1, "Memory Alloc Error");

        SetXYlink(pt);
        if(n==0) head = pt;
        pt->x1 = ptStart.x;
        pt->y1 = ptStart.y;
        pt->x2 = ptEnd.x;
        pt->y2 = ptEnd.y;
        MoveTo(hdc, pt->x1, pt->y1);
        LineTo(hdc, pt->x2, pt->y2);
        n++;
        return(TRUE);
    }
    else
        pt = head;
        while(pt)
        {
            MoveTo(hdc, pt->x1, pt->y1);
            LineTo(hdc, pt->x2, pt->y2);
            pt = pt->next;
        }
}

void SetXYlink(pt)              /* FILENAME: SETXYLNK.C */
LIST *pt;
{
    static LIST *last = NULL;

    if(!last)
        last = pt;
    else
        last->next=pt;
    pt->next = NULL;
    last = pt;
}


BOOL DrawXYcircle(hdc, ptStart, ptEnd)   /* FILENAME: DRWCIRCL.C */
HDC hdc;
POINT ptStart;
POINT ptEnd;
{
    int nDif;
    static RECT rc;
    static BOOL bStart = TRUE;

    if(ptStart.x == -1)
    {
        Ellipse(hdc, rc.left, rc.top, rc.right, rc.bottom);
        return(bStart=TRUE);
    }

    nDif = abs(ptStart.x - ptEnd.x);
    if(ptStart.y < ptEnd.y)
        ptEnd.y = ptStart.y + nDif;
    else
        ptEnd.y = ptStart.y - nDif;

    if(bStart)
        Ellipse(hdc, ptStart.x, ptStart.y, ptEnd.x, ptEnd.y);
    else
    {
        FillRect(hdc, &rc, GetStockObject(WHITE_BRUSH));
        Ellipse(hdc, ptStart.x, ptStart.y, ptEnd.x, ptEnd.y);
    }
    rc.left = ptStart.x;
    rc.top = ptStart.y;
    rc.right = ptEnd.x;
    rc.bottom = ptEnd.y;

    return(bStart=FALSE);
}

BOOL DrawXYbox(hdc, ptStart, ptEnd)   /* FILENAME: DRWBOX.C */
HDC hdc;
POINT ptStart;
POINT ptEnd;
{
    static RECT rc;
    static BOOL bStart = TRUE;

    if(ptStart.x == -1)
    {
        Rectangle(hdc, rc.left, rc.top, rc.right, rc.bottom);
        return(bStart=TRUE);
    }

    if(bStart)
        Rectangle(hdc, ptStart.x, ptStart.y, ptEnd.x, ptEnd.y);
    else
    {
        FillRect(hdc, &rc, GetStockObject(WHITE_BRUSH));
        Rectangle(hdc, ptStart.x, ptStart.y, ptEnd.x, ptEnd.y);
    }
    rc.left = ptStart.x;
    rc.top = ptStart.y;
    rc.right = ptEnd.x;
    rc.bottom = ptEnd.y;

    return(bStart=FALSE);
}

BOOL DrawXYrbox(hdc, ptStart, ptEnd)   /* FILENAME: DRWRBOX.C */
HDC hdc;
POINT ptStart;
POINT ptEnd;
{
    int nEllipse;
    static RECT rc;
    static BOOL bStart = TRUE;

 /* Check and see if this is a redraw. This is indicated by the -1
    value in the ptStart.x. If it is a redraw, use the values from
    the static RECT structure to redrawn the rounded box. */

    if(ptStart.x == -1)
    {
        RoundRect(hdc, rc.left, rc.top, rc.right, rc.bottom,
                                abs(rc.left - rc.right) / 4,
                                abs(rc.left - rc.right) / 4);
        return(bStart=TRUE);
    }

    nEllipse = abs(ptStart.x - ptEnd.x) / 4;

    if(bStart)
        RoundRect(hdc, ptStart.x, ptStart.y,
                    ptEnd.x, ptEnd.y, nEllipse, nEllipse);
    else
    {
        FillRect(hdc, &rc, GetStockObject(WHITE_BRUSH));
        RoundRect(hdc, ptStart.x, ptStart.y,
                    ptEnd.x, ptEnd.y, nEllipse, nEllipse);
    }

    rc.left = ptStart.x;
    rc.top = ptStart.y;
    rc.right = ptEnd.x;
    rc.bottom = ptEnd.y;

    return(bStart=FALSE);
}

BOOL DrawXYellipse(hdc, ptStart, ptEnd)   /* FILENAME: DRWELLIP.C */
HDC hdc;
POINT ptStart;
POINT ptEnd;
{
    static RECT rc;
    static BOOL bStart = TRUE;

    if(ptStart.x == -1)
    {
        Ellipse(hdc, rc.left, rc.top, rc.right, rc.bottom);
        return(bStart=TRUE);
    }

    if(bStart)
        Ellipse(hdc, ptStart.x, ptStart.y, ptEnd.x, ptEnd.y);
    else
    {
        FillRect(hdc, &rc, GetStockObject(WHITE_BRUSH));
        Ellipse(hdc, ptStart.x, ptStart.y, ptEnd.x, ptEnd.y);
    }
    rc.left = ptStart.x;
    rc.top = ptStart.y;
    rc.right = ptEnd.x;
    rc.bottom = ptEnd.y;

    return(bStart=FALSE);
}

BOOL DrawXYarc(hdc, pt)   /* FILENAME: DRWARC.C  */
HDC hdc;
POINT pt;
{

    static int i;
    static POINT ptArc[4];

    if(!hdc && i < 4)
    {
        ptArc[i] = pt;
        i++;
        return(FALSE);
    }
    if(hdc && i >= 4)
    {
        Arc(hdc, ptArc[0].x, ptArc[0].y,
                   ptArc[1].x, ptArc[1].y,
                   ptArc[2].x, ptArc[2].y,
                   ptArc[3].x, ptArc[3].y);

        i=0;
        return(TRUE);
    }
}

BOOL DrawXYchord(hdc, pt)   /* FILENAME: DRWCHORD.C */
HDC hdc;
POINT pt;
{

    static int i;
    static POINT ptChord[4];

    if(!hdc && i < 4)
    {
        ptChord[i] = pt;
        i++;
        return(FALSE);
    }
    if(hdc && i >= 4)
    {
        Chord(hdc, ptChord[0].x, ptChord[0].y,
                   ptChord[1].x, ptChord[1].y,
                   ptChord[2].x, ptChord[2].y,
                   ptChord[3].x, ptChord[3].y);

        i=0;
        return(TRUE);
    }
}

BOOL DrawXYpie(hdc, pt)   /* FILENAME: DRWPIE.C */
HDC hdc;
POINT pt;
{

    static int i;
    static POINT ptPie[4];

    if(!hdc && i < 4)
    {
        ptPie[i] = pt;
        i++;
        return(FALSE);
    }
    if(hdc && i >= 4)
    {
        Pie(hdc, ptPie[0].x, ptPie[0].y,
                 ptPie[1].x, ptPie[1].y,
                 ptPie[2].x, ptPie[2].y,
                 ptPie[3].x, ptPie[3].y);

        i=0;
        return(TRUE);
    }
}

BOOL DrawXYpline(hdc, pt)   /* FILENAME: DRWPLINE.C */
HDC hdc;
POINT pt;
{
    static int n;
    static POINT ptPline[50];

    if(pt.x != -1)
    {
        if(n > 50)
        {
            memset(ptPline, 0, sizeof(pt));
            n = 0;
            return(FALSE);
        }

        ptPline[n].x = pt.x;
        ptPline[n].y = pt.y;
        n++;
        return(TRUE);
    }
    else
    {
        Polyline(hdc, ptPline, n);
        return(TRUE);
    }
}

BOOL DrawXYpolygon(hdc, pt)   /* FILENAME: DRWPGON.C */
HDC hdc;
POINT pt;
{
    static int n;
    static POINT ptGon[50];

    if(pt.x != -1)
    {
        if(n > 50)
        {
            memset(ptGon, 0, sizeof(pt));
            n = 0;
            return(FALSE);
        }

        ptGon[n].x = pt.x;
        ptGon[n].y = pt.y;
        n++;
        return(TRUE);
    }
    else
    {
        Polygon(hdc, ptGon, n);
        return(TRUE);
    }
}

BOOL DrawXYtriangle(hdc, pt)   /* FILENAME: DRWTRI.C */
HDC hdc;
POINT pt;
{

    static int i;
    static POINT ptTri[3];

    if(!hdc && i < 3)
    {
        ptTri[i] = pt;
        i++;
        return(FALSE);
    }
    if(hdc && i >= 3)
    {
        MoveTo(hdc, ptTri[0].x, ptTri[0].y);
        LineTo(hdc, ptTri[1].x, ptTri[1].y);
        LineTo(hdc, ptTri[2].x, ptTri[2].y);
        LineTo(hdc, ptTri[0].x, ptTri[0].y);
        i=0;
        return(TRUE);
    }
}

void StatusBox(hwnd)           /* FILENAME: STATBOX.C */
HWND hwnd;
{
    HDC hdc;
    RECT rc;
    int nMenubar;
    HBRUSH hBrush;

    hdc = GetDC(hwnd);
    GetClientRect(hwnd, &rc);
    nMenubar = GetSystemMetrics(SM_CYMENU);
    rc.top = rc.bottom - nMenubar;
    SelectObject(hdc, hBrush=CreateSolidBrush(RGB(128,128,128)));
    Rectangle(hdc, rc.left, rc.top, rc.right, rc.bottom);
    SetBkColor(hdc, RGB(128,128,128));
    TextOut(hdc, rc.left + 15, rc.bottom - 17, (LPSTR)"Horizontal: ", 12);
    TextOut(hdc, rc.left + 140, rc.bottom - 17, (LPSTR)"Vertical: ", 10);
    TextOut(hdc, rc.left + 250, rc.bottom - 17, (LPSTR)"Window: ", 8);

    ReleaseDC(hwnd, hdc);
    DeleteObject(hBrush);
}

void UpdateXY(hwnd, hdc, pt)     /* FILENAME: UPDATEXY.C */
HWND hwnd;
HDC hdc;
POINT pt;
{
    char buf[4];
    BOOL bLocalDC = FALSE;
    RECT rc;

    if(!hdc)
    {
        hdc = GetDC(hwnd);
        bLocalDC = TRUE;
    }

    GetClientRect(hwnd, &rc);
    itoa(pt.x, buf, 10);
    if(pt.x < 10) strcat(buf, " ");
    if(pt.x < 100) strcat(buf, " ");
    strcat(buf, " ");
    SetBkColor(hdc, RGB(128,128,128));
    TextOut(hdc, rc.left + 90, rc.bottom - 17, (LPSTR)buf, 4);
    itoa(pt.y, buf, 10);
    if(pt.y < 10) strcat(buf, " ");
    if(pt.y < 100) strcat(buf, " ");
    strcat(buf, " ");
    TextOut(hdc, rc.left + 193, rc.bottom - 17, (LPSTR)buf, 4);
    if(bLocalDC)
        ReleaseDC(hwnd, hdc);
}

void UpdateXYwnd(hwnd, hdc)     /* FILENAME: UPXYWND.C */
HWND hwnd;
HDC hdc;
{
    char buf1[10], buf2[10];
    RECT rc;
    BOOL bLocalDC = FALSE;

    if(!hdc)
    {
        hdc = GetDC(hwnd);
        bLocalDC = TRUE;
    }

    GetClientRect(hwnd, &rc);
    itoa(rc.bottom - rc.top, buf1, 10);
    strcat(buf1, " x ");
    itoa(rc.right - rc.top, buf2, 10);
    strcat(buf1, buf2);
    SetBkColor(hdc, RGB(128,128,128));
    TextOut(hdc, rc.left + 312, rc.bottom - 17,
                          (LPSTR)buf1, strlen(buf1));
    if(bLocalDC)
        ReleaseDC(hwnd, hdc);
}

void ColorRGBbar(hwnd)        /* FILENAME: COLORBAR.C */
HWND hwnd;
{
    HDC hdc;
    RECT rc;
    HBRUSH hBrush[8];
    int i, nSquare;
    static COLORREF rgb[8] = { RGB(0, 0, 0), RGB(255, 255, 255),
                               RGB(255, 255, 0), RGB(255, 0, 255),
                               RGB(255, 0, 0), RGB(0, 255, 255),
                               RGB(0, 255, 0), RGB(0, 0, 255) };

    hdc = GetDC(hwnd);
    GetClientRect(hwnd, &rc);
    rc.left = rc.right - offRight;
    nSquare = (rc.bottom - offBottom - rc.top) / 8;
    if((rc.bottom - offBottom - rc.top) % 8)
        nSquare++;
    rc.bottom = 0;

    for(i=0; i < 8; i++)
    {
        rc.bottom += nSquare;
        rc.top = rc.bottom - nSquare;
        SelectObject(hdc, hBrush[i]=CreateSolidBrush(rgb[i]));
        Rectangle(hdc, rc.left, rc.top, rc.right, rc.bottom);
    }

    ReleaseDC(hwnd, hdc);
    for(i=0; i < 8; i++)
        DeleteObject(hBrush[i]);
}

BOOL GetPTrange(hwnd, pt)      /* FILENAME: PTINRANG.C */
HWND hwnd;
POINT pt;
{
    RECT rc;

    GetClientRect(hwnd, &rc);

    rc.left += offLeft;
    rc.top += offTop;
    rc.right -= offRight;
    rc.bottom -= offBottom;

    if(PtInRect(&rc, pt))
        return(TRUE);
    else
        return(FALSE);
}


DWORD GetXYcolor(hwnd, pt)     /* FILENAME: GETXYCOLR.C */
HWND hwnd;
POINT pt;
{
     HDC hdc;
     RECT rc;
     DWORD dwColor;

     hdc = GetDC(hwnd);
     GetClientRect(hwnd, &rc);

     if(pt.x <= rc.right - offRight
                      || pt.y >= rc.bottom - offBottom)
         return(rgbVal);

     dwColor = GetPixel(hdc, pt.x, pt.y);
     ReleaseDC(hwnd, hdc);
     return(dwColor);
}


/************************************************************************
 *                                                                      *
 *                          Bitmap Routines                             *
 *                                                                      *
 ************************************************************************/

BOOL DisplayBMload(hwnd, lpName, rc)    /* FILENAME: DISBMLOD.C */
HWND hwnd;
LPSTR lpName;
RECT rc;
{
       HDC hdc, hdcMem;
       HBITMAP hBitmap;

       if(!(hBitmap = LoadBitmap(hInst, lpName)))
           return(FALSE);

       hdc = GetDC(hwnd);
       hdcMem = CreateCompatibleDC(hdc);
       SelectObject(hdcMem, hBitmap);
       BitBlt(hdc, rc.left, rc.top, rc.right, rc.bottom,
                                     hdcMem, 0, 0, SRCCOPY);
       ReleaseDC(hwnd, hdc);
       DeleteDC(hdcMem);
       DeleteObject(hBitmap);

       return(TRUE);
}


BOOL DisplayBMmap(hwnd, lpName, rc)    /* FILENAME: DISBMMAP.C */
HWND hwnd;
LPSTR lpName;
RECT rc;
{
       HDC hdc, hdcMem;
       POINT ptBit;
       BITMAP bm;
       HBITMAP hBitmap;

       if(!(hBitmap = LoadBitmap(hInst, lpName)))
           return(FALSE);

       hdc = GetDC(hwnd);
       hdcMem = CreateCompatibleDC(hdc);
       SelectObject(hdcMem, hBitmap);
       SetMapMode(hdcMem, GetMapMode(hdc));
       GetObject(hBitmap, sizeof(BITMAP), (LPSTR)&bm);
       ptBit.x = bm.bmWidth;
       ptBit.y = bm.bmHeight;
       DPtoLP(hdc, &ptBit, 1);

       BitBlt(hdc, rc.left, rc.top, ptBit.x, ptBit.y,
                                     hdcMem, 0, 0, SRCCOPY);
       ReleaseDC(hwnd, hdc);
       DeleteDC(hdcMem);
       DeleteObject(hBitmap);
       return(TRUE);
}


BOOL StretchBMmap(hwnd, lpName, rc)    /* FILENAME: STHBMMAP.C */
HWND hwnd;
LPSTR lpName;
RECT rc;         /* Dest rect */
{

       HDC hdc, hdcMem;
       POINT ptBit;
       HBITMAP hBitmap;

       if(!(hBitmap = LoadBitmap(hInst, lpName)))
           return(FALSE);

       hdc = GetDC(hwnd);
       hdcMem = CreateCompatibleDC(hdc);
       SelectObject(hdcMem, hBitmap);
       MapBMpoint(hBitmap, hdc, hdcMem, &ptBit);

       StretchBlt(hdc, rc.left, rc.top, rc.right, rc.bottom,
                    hdcMem, 0, 0, ptBit.x, ptBit.y, SRCCOPY);

       DeleteDC(hdcMem);
       ReleaseDC(hwnd, hdc);
       DeleteObject(hBitmap);
       return(TRUE);
}

BOOL StretchBMclient(hwnd, lpName)    /* FILENAME: STHBMCLI.C */
HWND hwnd;
LPSTR lpName;
{

       HDC hdc, hdcMem;
       RECT rc;
       POINT ptBit;
       HBITMAP hBitmap;

       if(!(hBitmap = LoadBitmap(hInst, lpName)))
           return(FALSE);

       hdc = GetDC(hwnd);
       hdcMem = CreateCompatibleDC(hdc);
       SelectObject(hdcMem, hBitmap);
       MapBMpoint(hBitmap, hdc, hdcMem, &ptBit);
       CustomWNDrect(hwnd, &rc);   /* Or use GetClientRect */

       StretchBlt(hdc, rc.left, rc.top, rc.right, rc.bottom,
                     hdcMem, 0, 0, ptBit.x, ptBit.y, SRCCOPY);

       DeleteDC(hdcMem);
       ReleaseDC(hwnd, hdc);
       DeleteObject(hBitmap);
       return(TRUE);
}

void MapBMpoint(hBitmap, hdc, hdcMem, lpPt)    /* FILENAME: MAPBMPT.C */
HBITMAP hBitmap;
HDC hdc;
HDC hdcMem;
LPPOINT lpPt;
{
       BITMAP bm;

       SetMapMode(hdcMem, GetMapMode(hdc));
       GetObject(hBitmap, sizeof(BITMAP), (LPSTR)&bm);
       lpPt->x = bm.bmWidth;
       lpPt->y = bm.bmHeight;
       DPtoLP(hdc, lpPt, 1);

}

void DisplayBMmove(hwnd, hBitmap, pt)    /* FILENAME: DISBMMOV.C */
HWND hwnd;
HBITMAP hBitmap;
POINT pt;
{
       HDC hdc, hdcMem;
       POINT ptBit;

       hdc = GetDC(hwnd);
       hdcMem = CreateCompatibleDC(hdc);
       SelectObject(hdcMem, hBitmap);

       MapBMpoint(hBitmap, hdc, hdcMem, &ptBit);

       BitBlt(hdc, pt.x, pt.y, ptBit.x, ptBit.y,
                                     hdcMem, 0, 0, SRCCOPY);
       ReleaseDC(hwnd, hdc);
       DeleteDC(hdcMem);
}
